from .constants import DIRECTIVES
from checker.checker.src import doc_as_code

def check_traceability_links(needs, need, needs_flow_link_types, log):
    if not needs:
        return False
    allowed_links = DIRECTIVES.get(need['type'])
    if allowed_links is None or not allowed_links:
        return False
    for link_type in needs_flow_link_types:
        if need[link_type]:
            links = need[link_type]
            for link in links:
                target_need = needs[link]
                target_need_type = target_need['type']
                if target_need_type not in allowed_links:
                    log.warn(f"Directive {need['type']} cannot be associated "
                             f"with directive {target_need_type}. the allowed directive(s) of {need['type']} is/are "
                             f"{allowed_links}")
                    return True
    return False

##
#
# @rst
# :need: REQ_1

# .. req:: My #11 requirement
#     :id: REQ_11
#     :status: draft
#     :tags: example
#
#     This need is a requirement, and it includes a title, an ID, a Tag and this test as a description.
#
#     We didn't set the **ID** option here, so **Sphinx-Needs** will generate one for us.
#     But we have set a **link** to our previous requirement and have set the **status** option.
#     Also, we have enabled **collapse** to false to show all meta-data directly under the title.
# @endrst

class Helper:

    def __init__(self):
        print("hallo")

    ##
    #
    # @rst
    # .. req:: A more complex and highlighted requirement
    #   :id: REQ_3
    #   :status: open
    #   :tags: awesome, nice, great
    #   :links: REQ_1
    #   :layout: complete
    #   :style: red_border

    #   More columns for better data structure and a red border.
    # @endrst
    def fun_helper():
        return 0


# Software
# Architecture
SWT_ARCH_ARCHITECTURE = 'arch-sw'
SWT_ARCH_FUNCTIONAL_ARCHITECTURE = 'sw-if-def'
# Design
SWT_DSN_UNIT = 'sw-unit'
SWT_DSN_DETAILED_DESIGN = 'sw-dsgn'
SWT_DSN_MODEL = 'sw-mod'
# Requirement
SWT_REQ_REQUIREMENT = 'req-sw'
SWT_REQ_PARAMETER_AND_SIGNAL_DEFINITION = 'par-sig-def'
SWT_REQ_SOFT = 'sw-if-def'  # TODO

# System
# Architecture
SYS_ARCH_FUNCTIONAL = 'arch-sys-f'
SYS_ARCH_LOGICAL = 'arch-sys-l'
# Object
SYS_OBJ_CATALOG = 'obj-catalog'
SYS_OBJ_TYPE_CATALOG = 'obj-type-catalog'
# Requirement
SYS_REQ_REQUIREMENT = 'req-sys'
SYS_REQ_SPECIFICATION_SCENARIO = 'spec-scen'
SYS_REQ_ELICITED_SYSTEM = 'req-eli-sys'
# Stakeholder
SYS_SH_NEED = 'sh-need'
SYS_SH_REQUIREMENT = 'req-sh'

# Hardware
HARDWARE_ECU_REQUIREMENT = 'req-ecu'

# Software
# Architecture
SWT_ARCH_ARCHITECTURE_ALLOWED_LINKS = []
SWT_ARCH_FUNCTIONAL_ARCHITECTURE_ALLOWED_LINKS = []
# Design
SWT_DSN_UNIT_ALLOWED_LINKS = [SYS_SH_REQUIREMENT, SYS_REQ_ELICITED_SYSTEM, SYS_OBJ_TYPE_CATALOG,
                              SWT_DSN_DETAILED_DESIGN, SWT_DSN_MODEL]
SWT_DSN_DETAILED_DESIGN_ALLOWED_LINKS = [SYS_OBJ_TYPE_CATALOG, SWT_REQ_PARAMETER_AND_SIGNAL_DEFINITION,
                                         SYS_REQ_REQUIREMENT, SYS_REQ_REQUIREMENT, SWT_ARCH_FUNCTIONAL_ARCHITECTURE, SYS_SH_REQUIREMENT]
SWT_DSN_MODEL_ALLOWED_LINKS = []
# Requirement
SWT_REQ_REQUIREMENT_ALLOWED_LINKS = []
SWT_REQ_PARAMETER_AND_SIGNAL_DEFINITION_ALLOWED_LINKS = []
SWT_REQ_SOFT_ALLOWED_LINKS = []

# System
# Architecture
SYS_ARCH_FUNCTIONAL_ALLOWED_LINKS = []
SYS_ARCH_LOGICAL_ALLOWED_LINKS = []
# Object
SYS_OBJ_CATALOG_ALLOWED_LINKS = []
SYS_OBJ_TYPE_CATALOG_ALLOWED_LINKS = []
# Requirement
SYS_REQ_SPECIFICATION_SCENARIO_ALLOWED_LINKS = []
SYS_REQ_REQUIREMENT_ALLOWED_LINKS = [
    SYS_OBJ_CATALOG, SYS_OBJ_TYPE_CATALOG, SYS_SH_REQUIREMENT, HARDWARE_ECU_REQUIREMENT]
SYS_REQ_ELICITED_SYSTEM_ALLOWED_LINKS = []
# Stakeholder
SYS_SH_NEED_ALLOWED_LINKS = []
SYS_SH_REQUIREMENT_ALLOWED_LINKS = []

# Hardware
HARDWARE_ECU_REQUIREMENT_ALLOWED_LINKS = []

DIRECTIVES = {
    # Software
    # Architecture
    SWT_ARCH_ARCHITECTURE: SWT_ARCH_ARCHITECTURE_ALLOWED_LINKS,
    SWT_ARCH_FUNCTIONAL_ARCHITECTURE: SWT_ARCH_FUNCTIONAL_ARCHITECTURE_ALLOWED_LINKS,
    # Design
    SWT_DSN_UNIT: SWT_DSN_UNIT_ALLOWED_LINKS,
    SWT_DSN_DETAILED_DESIGN: SWT_DSN_DETAILED_DESIGN_ALLOWED_LINKS,
    SWT_DSN_MODEL: SWT_DSN_MODEL_ALLOWED_LINKS,
    # REQUIREMENT
    SWT_REQ_REQUIREMENT: SWT_REQ_REQUIREMENT_ALLOWED_LINKS,
    SWT_REQ_PARAMETER_AND_SIGNAL_DEFINITION: SWT_REQ_PARAMETER_AND_SIGNAL_DEFINITION_ALLOWED_LINKS,
    SWT_REQ_SOFT: SWT_REQ_SOFT_ALLOWED_LINKS,
    # System
    # Architecture
    SYS_ARCH_FUNCTIONAL: SYS_ARCH_FUNCTIONAL_ALLOWED_LINKS,
    SYS_ARCH_LOGICAL: SYS_ARCH_LOGICAL_ALLOWED_LINKS,
    # Object
    SYS_OBJ_CATALOG: SYS_OBJ_CATALOG_ALLOWED_LINKS,
    SYS_OBJ_TYPE_CATALOG: SYS_OBJ_TYPE_CATALOG_ALLOWED_LINKS,
    # Requirement
    SYS_REQ_REQUIREMENT: SYS_REQ_SPECIFICATION_SCENARIO_ALLOWED_LINKS,
    SYS_REQ_SPECIFICATION_SCENARIO: SYS_REQ_REQUIREMENT_ALLOWED_LINKS,
    SYS_REQ_ELICITED_SYSTEM: SYS_REQ_ELICITED_SYSTEM_ALLOWED_LINKS,
    # Stakeholder
    SYS_SH_NEED: SYS_SH_NEED_ALLOWED_LINKS,
    SYS_SH_REQUIREMENT: SYS_SH_REQUIREMENT_ALLOWED_LINKS,
    # Hardware
    HARDWARE_ECU_REQUIREMENT: HARDWARE_ECU_REQUIREMENT_ALLOWED_LINKS
}



from setuptools import find_packages, setup
setup(
    name='malsabbagh-checker-multiple-two',
    packages=find_packages(),
    version='0.2.0',
    description='Python links checker',
    author='Mohamed',
    license='MIT',
    # entry_points = {
    #     'console_scripts': [
    #         'command-name = checker.doc_as_code:main',
    #     ],              
    # },
    scripts=['scripts/doc-as-code']
    # install_requires=[],
    # setup_requires=['pytest-runner==6.0.0'],
    # tests_require=['pytest==7.2.1'],
    # test_suite='tests',
)



import subprocess
import os, sys
from checker import package_dir

def main():
    args = sys.argv[1:]
    if len(args) < 2:
        print("args must have length = 2") # TODO Exception
        return
    
    source_dir = os.path.join(os.getcwd(), args[0])
    output_dir = os.path.join(os.getcwd(), args[1])
    doxygen_output_dir = os.path.join(output_dir, "_doxygen")
    os.chdir(package_dir)
    subprocess.run(["make", "doxygen"], env=dict(os.environ, SOURCE_DIR=source_dir, DOXYGEN_OUTPUT_DIR=doxygen_output_dir))
    
    print("successful")